<?php
 /**
 * 
 *  2016 E-goi API V2.  
 *  
 */
class EgoiAPIV2 {
	
    const PLUGIN_KEY = 'e4bf5263e34400a6eda8a8d612e9b44b';
    const API_URL = 'https://api.e-goi.com/v2/soap.php?wsdl';
    public $apiKey;
    
    /** SOAP CLIENT INSTANCE **/
    private $_client;
    
    public function __construct() {
    	$this->apiKey = Configuration::get('egoi_api_key');
    	$this->_client = new SoapClient(self::API_URL);
    }
	
	public function getApiKey() {
		return $this->apiKey?$this->apiKey:Configuration::get('egoi_api_key');
	}
	
	private function getSoapParams() {
		return array(
			'apikey' => $this->getApiKey(),
			'plugin_key' => self::PLUGIN_KEY
		);
	}
	
	public function getUserData() {
		return $this->_client->getUserData($this->getSoapParams());
	}
	
	public function getClientData() {
		return $this->_client->getClientData($this->getSoapParams());
	}
	
	/*
	* 
	* Create List
	*
	*/
	public function createList($name, $lang) {
		
		$params = array(
			'nome' => $name,
			'idioma_lista' => $lang
		);
		
		$result = $this->_client->createList(array_merge($this->getSoapParams(), $params));
		if(isset($result['ERROR'])) {
			return $result['ERROR'];
		} else {
			$id = $result['LIST_ID'];
			return $id;
		}
	}
	
	/*
	*
	* Get Lists
	*
	*/
	public function getLists() {

		return $this->_client->getLists($this->getSoapParams());
	}

	/*
	*
	* Get Forms from List
	*
	*/
	public function getForms($listId = '', $option = false) {
		
		$forms = $this->_client->getForms(array_merge($this->getSoapParams(), array('listID' => $listId)));

		if($option){
			$array = array();
			foreach($forms as $key => $value){
				$array['url'] = $value['url'];
				$array['hash'] = $value['hash'];
			}
			return $array;
			
		}
		return $forms;
	}
	
	/*
	*
	* Get All Subscribers from List
	*
	*/
	public function getSubscribersFromListId($listId, $option = '') {

		$subs = $this->_client->subscriberData(array_merge($this->getSoapParams(), array('listID' => $listId, 'subscriber' => 'all_subscribers')));
		if($option) {
			$array_subs = array();
			foreach($subs['subscriber'] as $key => $subscriber){	
				if($subscriber['STATUS'] != '2'){
					$array_subs[] = $subscriber;
				}
			}
			$total_subs = count($array_subs);
			return $total_subs;
		}else{
			return $subs;
		}
	}

	public function getSubscriber($listId, $email) {

		$subs = $this->_client->subscriberData(array_merge($this->getSoapParams(), array('listID' => $listId, 'subscriber' => $email)));
		return $subs;
	}

	public function getAllSubscribers($listId) {

		$subs = $this->_client->subscriberData(array_merge($this->getSoapParams(), array('listID' => $listId, 'subscriber' => 'all_subscribers')));
		$count = 0;
		foreach($subs['subscriber'] as $key => $subscriber){
			if($subscriber['STATUS'] != '2'){
				$count++;
			}
		}

		return $count;
	}

	
	public function addSubscriber(CustomerCore $customer, $id_list, $formID = '', $extraFields) {
		
		$params = array( 
			'listID' => $id_list,
			'subscriber' => $customer->id,
			'status' => '1',
			'from' => '',
			'lang' => Language::getIsoById($customer->id_lang),
		    'email' => $customer->email,
			'validate_email' => '0',
		    'cellphone' => '',
		    'telephone' => '',
		    'fax' => '',
		    'formID' => $formID?$formID:'',
		    'first_name' => $customer->firstname,
		    'last_name' => $customer->lastname,
		    'birth_date' => ($customer->birthday) ? date('Y-m-d', strtotime($customer->birthday)):'',
		);
		
		
		if($extraFields AND count ($extraFields)> 0)
			$params = array_merge($params, $extraFields);
		
		$result = $this->_client->addSubscriber(array_merge($this->getSoapParams(), $params));
		
		if (isset($result['ERROR']))
			return false;
		else {
			$id = $result['UID'];
			return $id;
		}
	}

	public function addSubscriberBulk($id_list, $subscribers = array(), $tag = false) {
		
		$params = array( 
			'listID' 		=> $id_list,
    		'compareField' 	=> 'email',
    		'operation' 	=> 2,
    		'subscribers' 	=> $subscribers,
    		'tags' 			=> array($tag)
		);
		
		$result = $this->_client->addSubscriberBulk(array_merge($this->getSoapParams(), $params));
		return true;
	}

	public function addSubscriberForm($list, $email, $fname = '', $lname = '', $number = '', $formID = '', $tag = '', $fields = array()) {
		
		if($number){
			if(substr($number, 0, 1) != '9'){
				$tel = $number;
			}else{
				$phone = $number;
			}
		}

		$params = array(
			'listID' => $list,
			'subscriber' => $email,
		    'email' => $email,
			'validate_email' => '0',
		    'cellphone' => $phone ? $phone : '',
		    'telephone' => $tel ? $tel : '',
		    'first_name' => $fname ? $fname : '',
		    'last_name' => $lname ? $lname : '',
		    'formID' => $formID ? $formID : '',
		    'tags' => array($tag ? $tag : ''),
		    'status' => 1
		);

		if(!empty($fields)){
			foreach ($fields as $key => $value) {
				if($key == 'birth_date'){
					$value = date('Y-m-d', strtotime($value));
				}
				$params[$key] = $value;
			}
		}
		
		$result = $this->_client->addSubscriber(array_merge($this->getSoapParams(), $params));
		return $result;
	}

	public function editSubscriber($list, $email, $fname = '', $lname = '', $fields = array()) {
		
		$params = array(
			'listID' => $list,
			'subscriber' => $email,
			'email' => $email,
			'first_name' => $fname,
			'last_name' => $lname
		);

		if(!empty($fields)){
			foreach ($fields as $key => $value) {
				if($key == 'birth_date'){
					$value = date('Y-m-d', strtotime($value));
				}
				$params[$key] = $value;
			}
		}

		$result = $this->_client->editSubscriber(array_merge($this->getSoapParams(), $params));
		return $result;
	}

	public function removeSubscriber($list, $email) {
		
		$params = array(
			'listID' => $list,
			'subscriber' => $email
		);
		
		$result = $this->_client->removeSubscriber(array_merge($this->getSoapParams(), $params));
		return $result;
	}
	
	public function createExtraField($id_list, $name, $type) {
		
		//ALL REQUIRED
		$params = array( 
			'listID' => $id_list,
			'name' => $name,
			'type' => $type
		);
		
		$result = $this->_client->addExtraField(array_merge($this->getSoapParams(), $params));
		
		if (isset($result['ERROR']))
			return false;
		else {
			$id = $result['NEW_ID'];
			return $id;
		}
	}

	public function getMappedFields() {

		$sql = "SELECT * FROM "._DB_PREFIX_."egoi_map_fields order by id DESC";
		$rq = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        return $rq;
	}

	public function getFieldMap($name = false, $field = false){
		
		if($field){
			$sql = "SELECT * FROM "._DB_PREFIX_."egoi_map_fields WHERE ps='".pSQL($field)."'";
		}else{
			$sql = "SELECT * FROM "._DB_PREFIX_."egoi_map_fields WHERE egoi='".pSQL($name)."'";
		}
        $rq = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        return $rq[0]['egoi'];
	}

	public function getExtraFields($listID) {

		$params = array(
			'listID' => $listID
		);
       	
       	$result_client = $this->_client->getExtraFields(array_merge($this->getSoapParams(), $params));

     	$extra_fields = $result_client['extra_fields'];
		return $extra_fields;
		
	}

	public function addTag($name) {
		
		$params = array( 
			'name' => $name,
		);
		$result = $this->_client->addTag(array_merge($this->getSoapParams(), $params));
		return $result;
	}

	public function getTags() {
			
		$result = $this->_client->getTags(array_merge($this->getSoapParams()));
		$tags = $result['TAG_LIST'];
		/*foreach($tags as $value){
			if($value['NAME'] == 'subscriber'){
				$tag[] = $value['ID']);
			}
		}*/
		return $tags;
	}
	
	
}